/********************************************************************
 * klist.c         20-FEB-1992            Y.Takeuchi (T.I.T.)       *
 *                                                                  *
 * K2917's list processing                                          *
 * for SPARC cc device driver  K2917  K3922                         *
 *                                                                  *
 * 26-NOV-1992 Y.Takeuchi    modify                                 *
 *  5-MAR-1993 Y.Takeuchi    modify for unified UNIX camlib         *
 ********************************************************************/

kladd(cc, name)
struct cc_device *cc;
char *name;
{
    register struct K_REG *k = cc->k;
    register int i, code;

    cc->status = 0;

    /* same name check */
    for (i = 0; i < ptr_dir; i++)
	if (strcmp(name, klist[i].name) == 0)
	    return CC_STA_KLIST_SAMENAME;

    if (ptr_dir >= CC_KLIST_MAX_DIR)
        return CC_STA_KLIST_MEMORYFULL;

    /* add */
    k->cma = ptr_cma;
    klist[ptr_dir].cma = ptr_cma;
    klist[ptr_dir].wc = 0;

    cc->ptr_list = list_area;
    while (cc->status == 0) {
	if (ptr_cma >= 0xFFFF - 4)
	    return CC_STA_KLIST_MEMORYFULL;
	code = 0x00FF & cc->ptr_list[0] + 0x60;
	instruction_call = instruction[code];
	cc->status = (*instruction_call)(cc);
	next_instruction(cc);
    }

    strcpy(klist[ptr_dir].name, name);
    ptr_dir++;

    return 0;
}


kldel(name)
char *name;
{
    register int i;

    /* same name check */
    for (i = 0; i < ptr_dir; i++)
	if (strcmp(name, klist[i].name) == 0) {
	    klist[i].name[0] = '\0';
	    return 0;
	}

    return CC_STA_KLIST_NONAME;
}


klexe(cc, name)
struct cc_device *cc;
char *name;
{
    register dev_t dev = cc->dev;
    register struct uio *uio = cc->uio;
    register struct buf *bp = &ccbuf;
    register int *retlen = (int *)cc->ptr_udata;
    register int i;

    cc->klist = CC_KLIST_YES;

    /* same name check */
    for (i = 0; i < ptr_dir; i++)
	if (strcmp(name, klist[i].name) == 0)
	    break;
    if (i >= ptr_dir)
        return CC_STA_KLIST_NONAME;

    now_cma = klist[i].cma;
    now_wc  = klist[i].wc;

    /* first 2words for retlen */
    uio->uio_iov->iov_base = (caddr_t)(cc->ptr_udata + 2);
    uio->uio_iov->iov_len  = klist[i].wc * 2;
    uio->uio_iovcnt = 1;
    uio->uio_offset = 0;

    /* Kinetic 2917 list is now READ ONLY. */
    physio(ccstrategy, bp, dev, B_READ, ccminphys, uio);

    *retlen = cc->retlen;

    return bp->b_error;
}


klsave(cc, data)
struct cc_device *cc;
u_short *data;
{
    register struct K_REG *k = cc->k;
    register int i, j, ptr = 1;
    register u_short save;

    for (i = 0; i < ptr_dir; i++)
	if (klist[i].name[0] != '\0') {

	    /* save list name */
	    j = 0;
	    while (klist[i].name[j] != '\0')
	        data[ptr++] = klist[i].name[j++];
	    data[ptr++] = '\0';

	    /* save data word counts */
	    data[ptr++] = klist[i].wc;

	    /* save list */
	    k->cma = klist[i].cma;
	    while ((save = k->cmr) != HALT)
	        data[ptr++] = save;
	    data[ptr++] = HALT;
	}

    data[0] = ptr - 1;

    return 0;
}


klload(cc, data)
struct cc_device *cc;
u_short *data;
{
    register struct K_REG *k = cc->k;
    register int j, ptr = 1;

    while (ptr <= data[0]) {
	/* load list name */
	j = 0;
	while (data[ptr] != '\0')
	    klist[ptr_dir].name[j++] = data[ptr++];
	klist[ptr_dir].name[j] = data[ptr++];
	
	/* load data word counts */
	  klist[ptr_dir].wc = data[ptr++];
	
	/* load list */
	k->cma = ptr_cma; 
	klist[ptr_dir].cma = ptr_cma;
	while (data[ptr] != HALT) {
	    k->cmr = data[ptr++];
	    ptr_cma++;
	}
	k->cmr = data[ptr++];
	ptr_cma++;

	ptr_dir++;
    }

    return 0;
}




/************************************************
 * K2917 list                                   *
 ************************************************/
kl_stop(cc)
struct cc_device *cc;
{
    register struct K_REG *k = cc->k;

    k->cmr = HALT;
    return CC_STA_LIST_STOP;
}

kl_read16(cc)
struct cc_device *cc;
{
    register struct K_REG *k = cc->k;

    k->cmr = AD | BIT16 | (cc->cur_crate << 8);
    k->cmr = cc->ptr_list[1];
    klist[ptr_dir].wc += 1;
    ptr_cma += 2;
    return 0;
}

kl_read24(cc)
struct cc_device *cc;
{
    register struct K_REG *k = cc->k;

    k->cmr = AD | BIT24 | (cc->cur_crate << 8);
    k->cmr = cc->ptr_list[1];
    klist[ptr_dir].wc += 2;
    ptr_cma += 2;
    return 0;
}

kl_write16(cc)
struct cc_device *cc;
{
    register struct K_REG *k = cc->k;

    k->cmr = AD | INLINE | BIT16 | (cc->cur_crate << 8);
    k->cmr = cc->ptr_list[1];
    k->cmr = cc->ptr_list[2];
    k->cmr = 0;
    ptr_cma += 4;
    return 0;
}

kl_write24(cc)
struct cc_device *cc;
{
    register struct K_REG *k = cc->k;

    k->cmr = AD | INLINE | BIT24 | (cc->cur_crate << 8);
    k->cmr = cc->ptr_list[1];
    k->cmr = cc->ptr_list[2];
    k->cmr = cc->ptr_list[3];
    ptr_cma += 4;
    return 0;
}

kl_ndt(cc)
struct cc_device *cc;
{
    register struct K_REG *k = cc->k;

    k->cmr = AD | BIT16 | (cc->cur_crate << 8);
    k->cmr = cc->ptr_list[1];
    ptr_cma += 2;
    return 0;
}

kl_qs16(cc)
struct cc_device *cc;
{
    register struct K_REG *k = cc->k;

    k->cmr = AD | BLOCK | BIT16 | QSTOP | (cc->cur_crate << 8);
    k->cmr = cc->ptr_list[1];
    k->cmr = - cc->ptr_list[2];
    k->cmr = 0xFFFF;
    klist[ptr_dir].wc += cc->ptr_list[2];
    ptr_cma += 4;
    return 0;
}

kl_qs24(cc)
struct cc_device *cc;
{
    register struct K_REG *k = cc->k;

    k->cmr = AD | BLOCK | BIT24 | QSTOP | (cc->cur_crate << 8);
    k->cmr = cc->ptr_list[1];
    k->cmr = - cc->ptr_list[2] * 2;
    k->cmr = 0xFFFF;
    klist[ptr_dir].wc += cc->ptr_list[2] * 2;
    ptr_cma += 4;
    return 0;
}

kl_as16(cc)
struct cc_device *cc;
{
    register struct K_REG *k = cc->k;

    k->cmr = AD | BLOCK | BIT16 | QSCAN | (cc->cur_crate << 8);
    k->cmr = cc->ptr_list[1];
    k->cmr = - cc->ptr_list[2];
    k->cmr = 0xFFFF;
    klist[ptr_dir].wc += cc->ptr_list[2];
    ptr_cma += 4;
    return 0;
}

kl_as24(cc)
struct cc_device *cc;
{
    register struct K_REG *k = cc->k;

    k->cmr = AD | BLOCK | BIT24 | QSCAN | (cc->cur_crate << 8);
    k->cmr = cc->ptr_list[1];
    k->cmr = - cc->ptr_list[2] * 2;
    k->cmr = 0xFFFF;
    klist[ptr_dir].wc += cc->ptr_list[2] * 2;
    ptr_cma += 4;
    return 0;
}

kl_igq16(cc)
struct cc_device *cc;
{
    register struct K_REG *k = cc->k;

    k->cmr = AD | BLOCK | BIT16 | QIGNO | (cc->cur_crate << 8);
    k->cmr = cc->ptr_list[1];
    k->cmr = - cc->ptr_list[2];
    k->cmr = 0xFFFF;
    klist[ptr_dir].wc += cc->ptr_list[2];
    ptr_cma += 4;
    return 0;
}

kl_igq24(cc)
struct cc_device *cc;
{
    register struct K_REG *k = cc->k;

    k->cmr = AD | BLOCK | BIT24 | QIGNO | (cc->cur_crate << 8);
    k->cmr = cc->ptr_list[1];
    k->cmr = - cc->ptr_list[2] * 2;
    k->cmr = 0xFFFF;
    klist[ptr_dir].wc += cc->ptr_list[2] * 2;
    ptr_cma += 4;
    return 0;
}
