/********************************************************************
 * camlib.c             20-FEB-1992            Y.Takeuchi (T.I.T.)  *
 *                                                                  *
 * original: camlib.c on SUN4 by Y.Yasu (KEK)                       *
 * for SUN-SPARC SF-VME K2917 K3922                                 *
 *                                                                  *
 * 24-JUN-1992 Y.Takeuchi    add checking file pointer              *
 * 30-NOV-1992 Y.Takeuchi    delete signal subroutines              *
 *  5-MAR-1993 Y.Takeuchi    modify for unified UNIX camlib         *
 * 16-JUN-1993 Y.Takeuchi    modify CAMACW()                        *
 *  3-SEP-1993 Y.Takeuchi    fixed bug at CAMACW()                  *
 ********************************************************************/
#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>
#include <sys/uio.h>
#include <sys/file.h>
#include <sys/errno.h>
#include <sys/ioctl.h>
#include "cc_config.h"
#include "cc_common.h"
#include "cc.h"
#include "k2917.h"
#include "camlib.h"

static int cc_path;
static struct iovec vec[3];
static struct cc_iosb iosb;
static struct cc_message message;
static int dummy;
static file_pointer;


check_file_pointer(bytecount)
int bytecount;
{
    file_pointer += bytecount;
    if (file_pointer > 0x10000000) {
	lseek(cc_path, SEEK_SET, 0);
	file_pointer = 0;
    }
    return 0;
}


CAMOPN()
{
    if ((cc_path = open("/dev/cc", O_RDWR)) == -1)
        return ENODEV;
    file_pointer = 0;
    return 0;
}


CAMCLS()
{
    return close(cc_path);
}


CRESET()
{
    int dummy;
    return ioctl(cc_path, CCIOC_RESET, &dummy);
}


CSETCR(crate)
int crate;
{
    return ioctl(cc_path, CCIOC_SET_CRATE, &crate);
}


CSETBR(branch)
int branch;
{
    return ioctl(cc_path, CCIOC_SET_BRANCH, &branch);
}


CGENZ()
{
    int status, i, q, x;

    status = CAMAC(NAF(30, 0, 1), &i, &q, &x);
    if (status != 0)
	return status;
    i |= 1;
    return CAMAC(NAF(30, 0, 17), &i, &q, &x);
}


CGENC()
{
    int status, i, q, x;

    status = CAMAC(NAF(30, 0, 1), &i, &q, &x);
    if (status != 0)
	return status;
    i |= 2;
    return CAMAC(NAF(30, 0, 17), &i, &q, &x);
}


CSETI()
{
    int status, i, q, x;

    status = CAMAC(NAF(30, 0, 1), &i, &q, &x);
    if (status != 0)
	return status;
    i |= 4;
    return CAMAC(NAF(30, 0, 17), &i, &q, &x);
}


CREMI()
{
    int status, i, q, x;

    status = CAMAC(NAF(30, 0, 1), &i, &q, &x);
    if (status != 0)
	return status;
    i &= ~4;
    return CAMAC(NAF(30, 0, 17), &i, &q, &x);
}


CAMACW(naf, dat, q, x)
int naf, *q, *x;
u_short *dat;
{
    register int status;
    u_short qx;

    message.command = (u_short)CC_CMD_DOSINGLE;
    message.mode = (u_short)BIT16;
    message.naf = (u_short)naf;
    message.ptr_data = (u_short *)dat;
    message.ptr_qx = (u_short *)&qx;
    vec[0].iov_base = (caddr_t)&message;
    vec[0].iov_len = sizeof(struct cc_message);
    if ((status = writev(cc_path, vec, 1)) != 0)
        return status;
    *x = ((qx & NOX) == 0) ? 1 : 0;
    *q = ((qx & NOQ) == 0) ? 1 : 0;

    check_file_pointer(2);

    return (qx >> 12);
}


CAMAC(naf, dat, q, x)
int naf, *dat, *q, *x;
{
    register int status;
    u_short qx;

    message.command = (u_short)CC_CMD_DOSINGLE;
    message.mode = (u_short)BIT24;
    message.naf = (u_short)naf;
    message.ptr_data = (u_short *)dat;
    message.ptr_qx = (u_short *)&qx;
    vec[0].iov_base = (caddr_t)&message;
    vec[0].iov_len = sizeof(struct cc_message);
    if ((status = writev(cc_path, vec, 1)) != 0)
        return status;
    *x = ((qx & NOX) == 0) ? 1 : 0;
    *q = ((qx & NOQ) == 0) ? 1 : 0;

    check_file_pointer(2);

    return (qx >> 12);
}


CELAM(mask)
int mask;
{
    return ioctl(cc_path, CCIOC_ENABLE_LAM, &mask);
}


CDLAM()
{
    int dummy;

    return ioctl(cc_path, CCIOC_DISABLE_LAM, &dummy);
}


CWLAM(timeout)
int timeout;
{
    errno = 0;
    ioctl(cc_path, CCIOC_WAIT_LAM, &timeout);
    return errno; 
}


CDMAW(mode, naf, buf, len, retlen, err)
int mode, naf, len, *retlen, *err;
u_short *buf;
{
    register int status;

    message.command = (u_short)CC_CMD_DOBLOCK;
    message.mode = (u_short)((mode << 3) | BLOCK | BIT16);
    message.naf = (u_short)naf;
    message.ptr_iosb = (struct cc_iosb *)&iosb;
    vec[0].iov_base = (caddr_t)&message;
    vec[0].iov_len = sizeof(struct cc_message);
    vec[1].iov_base = (caddr_t)buf;
    vec[1].iov_len = len * sizeof(u_short);
/*
    if ((status = writev(cc_path, vec, 2)) != 0)
        return status;
*/
    writev(cc_path, vec, 2);
    *retlen = iosb.ret_length;
    *err = iosb.status;

    check_file_pointer(*retlen);

    return 0;
}


CDMAL(mode, naf, buf, len, retlen, err)
int mode, naf, len, *retlen, *err;
int *buf;
{
    register int status, i;

    message.command = (u_short)CC_CMD_DOBLOCK;
    message.mode = (u_short)((mode << 3) | BLOCK | BIT24);
    message.naf = (u_short)naf;
    message.ptr_iosb = (struct cc_iosb *)&iosb;
    vec[0].iov_base = (caddr_t)&message;
    vec[0].iov_len = sizeof(struct cc_message);
    vec[1].iov_base = (caddr_t)buf;
    vec[1].iov_len = len * sizeof(int);
/*
    if ((status = writev(cc_path, vec, 2)) != 0)
        return status;
*/
    writev(cc_path, vec, 2);
    *retlen = iosb.ret_length;
    *err = iosb.status;

    check_file_pointer(*retlen);

    return 0;
}


CLEWAI(list, lenlis, buf, lenbuf, iiosb)
u_short *list, *buf;
int lenlis, lenbuf;
struct cc_iosb *iiosb;
{
    register int status;

    message.command = (u_short)CC_CMD_LOADDOLIST;
    message.ptr_iosb = (struct cc_iosb *)iiosb;
    vec[0].iov_base = (caddr_t)&message;
    vec[0].iov_len = sizeof(struct cc_message);
    vec[1].iov_base = (caddr_t)list;
    vec[1].iov_len = lenlis * sizeof(u_short);
    vec[2].iov_base = (caddr_t)buf;
    vec[2].iov_len = lenbuf * sizeof(u_short);
    status = writev(cc_path, vec, 3);

    check_file_pointer(lenlis + iiosb->ret_length);

    return status;
}


CEWAI(buf, lenbuf, iiosb)
u_short *buf;
int lenbuf;
struct cc_iosb *iiosb;
{
    register int status;

    message.command = (u_short)CC_CMD_DOLIST;
    message.ptr_iosb = (struct cc_iosb *)iiosb;
    vec[0].iov_base = (caddr_t)&message;
    vec[0].iov_len = sizeof(struct cc_message);
    vec[1].iov_base = (caddr_t)buf;
    vec[1].iov_len = lenbuf * sizeof(u_short);
    status = writev(cc_path, vec, 2);

    check_file_pointer(iiosb->ret_length);
/*
    return status;
*/
    return (0);
}


CLLOAD(list, lenlis, err)
u_short *list;
int lenlis;
int *err;
{
    register int status;

    *err = 0;
    message.command = (u_short)CC_CMD_LOADLIST;
    message.ptr_iosb = (struct cc_iosb *)&iosb;
    vec[0].iov_base = (caddr_t)&message;
    vec[0].iov_len = sizeof(struct cc_message);
    vec[1].iov_base = (caddr_t)list;
    vec[1].iov_len = lenlis * sizeof(u_short);
    status = writev(cc_path, vec, 2);

    check_file_pointer(lenlis);

    return status;
}


CRLIST(fname, list, lenlis, retlen, err)
char *fname;
u_short *list;
int lenlis, *retlen, *err;
{
    register FILE *fp;
    int size, a, b, c, d;

    if ((fp = fopen(fname, "r")) == NULL) {
	*err = EFAULT;
	return EFAULT;
    }
    (void)fscanf(fp, "%X", &size);
    if (size > lenlis) {
	*err = -1;
	return -1;
    }
    *retlen = size;
    while (size > 0) {
	(void)fscanf(fp, "%X%X%X%X", &a, &b, &c, &d);
	*list++ = a;
	*list++ = b;
	*list++ = c;
	*list++ = d;
        size -= 4;
    }
    *err = 0;
    return fclose(fp);
}


CKINIT()
{
    return ioctl(cc_path, CCIOC_KINIT, &dummy);
}


CKADD(name, list, lenlis)
char *name;
u_short *list;
int lenlis;
{
    register int status;

    message.command = (u_short)CC_CMD_ADDKLIST;
    message.klname = name;
    vec[0].iov_base = (caddr_t)&message;
    vec[0].iov_len = sizeof(struct cc_message);
    vec[1].iov_base = (caddr_t)list;
    vec[1].iov_len = lenlis * sizeof(u_short);
    status = writev(cc_path, vec, 2);

    check_file_pointer(lenlis);

    return status;
}


CKDEL(name)
char *name;
{
    register int status;

    message.command = (u_short)CC_CMD_DELKLIST;
    message.klname = name;
    vec[0].iov_base = (caddr_t)&message;
    vec[0].iov_len = sizeof(struct cc_message);
    status = writev(cc_path, vec, 1);

    check_file_pointer(sizeof(vec));

    return status;
}


CKEXEC(name, buf, lenbuf, iiosb)
char *name;
u_short *buf;
int lenbuf;
struct cc_iosb *iiosb;
{
    register int status;

    message.command = (u_short)CC_CMD_EXEKLIST;
    message.klname = name;
    vec[0].iov_base = (caddr_t)&message;
    vec[0].iov_len = sizeof(struct cc_message);
    vec[1].iov_base = (caddr_t)buf;
    vec[1].iov_len = lenbuf * sizeof(u_short);
    status = writev(cc_path, vec, 2);

    check_file_pointer(iiosb->ret_length);

    return status;
}


CKSAVE(fname)
char *fname;
{
    register int fd;
    u_short data[8000];

    message.command = (u_short)CC_CMD_SAVEKLIST;
    message.ptr_iosb = (struct cc_iosb *)data;
    vec[0].iov_base = (caddr_t)&message;
    vec[0].iov_len = sizeof(struct cc_message);
    writev(cc_path, vec, 1);
    fd = creat(fname, 0666);
    write(fd, &data[0], 2);
    write(fd, &data[1], data[0] * 2);
    close(fd);

    check_file_pointer(8000);

    return 0;
}


CKLOAD(fname)
char *fname;
{
    register int fd;
    u_short data[8000];

    fd = open(fname, O_RDONLY);
    read(fd, &data[0], 2);
    read(fd, &data[1], data[0] * 2);
    close(fd);

    message.command = (u_short)CC_CMD_LOADKLIST;
    message.ptr_iosb = (struct cc_iosb *)data;
    vec[0].iov_base = (caddr_t)&message;
    vec[0].iov_len = sizeof(struct cc_message);
    writev(cc_path, vec, 1);

    check_file_pointer(8000);

    return 0;
}
